@@ -1,4 +1,4 @@
-// Copyright © 2011, 2012, 2014, 2015 Richard Kettlewell.
+// Copyright © 2011, 2012, 2014-17 Richard Kettlewell.
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -118,6 +118,37 @@ void ConfBase::write(std::ostream &os, int step, bool verbose) const {
   if(hookTimeout)
     os << indent(step) << "hook-timeout " << hookTimeout << '\n';
   d(os, "", 0);
+
+  d(os, "# rsync command", step);
+  d(os, "# rsync-command COMMAND", step);
+  os << indent(step) << "rsync-command " << quote(rsyncCommand) << '\n';
+  d(os, "", 0);
+
+  d(os, "# rsync base options", step);
+  d(os, "# rsync-base-options OPTION ...", step);
+  os << indent(step) << "rsync-base-options";
+  for(auto &opt: rsyncBaseOptions)
+    os << " " << quote(opt);
+  os << "\n";
+  d(os, "", 0);
+
+  d(os, "# rsync extra options", step);
+  d(os, "# rsync-extra-options OPTION ...", step);
+  os << indent(step) << "rsync-extra-options";
+  for(auto &opt: rsyncExtraOptions)
+    os << " " << quote(opt);
+  os << "\n";
+  d(os, "", 0);
+
+  // TODO hacky way of managing {toplevel,host}-only directives
+  if(what() != "volume") {
+    d(os, "# Host check behavior", step);
+    d(os, "#  host-check ssh", step);
+    d(os, "#  host-check always-up", step);
+    d(os, "#  host-check command COMMAND ...", step);
+    os << indent(step) << "host-check " << quote(hostCheck) << '\n';
+    d(os, "", 0);
+  }
 }
 
 void ConfBase::describe(std::ostream &os,
