@@ -48,6 +48,11 @@ public:
    */
   Date(time_t when);
 
+  /** @brief Assignment operator
+   * @param dateString Date in YYYY-MM-DD format
+   */
+  Date &operator=(const std::string &dateString);
+
   /** @brief Different between two dates in days
    * @param that Other date
    */
@@ -91,11 +96,29 @@ public:
    */
   bool operator!=(const Date &that) const { return !(*this == that); }
 
+  /** @brief Increment date
+   * @return Next day
+   */
+  Date &operator++();
+
+  /** @brief Advance by 1 month
+   * @return @c *this
+   *
+   * If the day ends up outside the month it is clipped to the last day.
+   */
+  Date &addMonth();
+
   /** @brief Convert to string
    * @return Date in "YYYY-MM-DD" format
    */
   std::string toString() const;         // YYYY-MM-DD
 
+  /** @brief Convert to a string using a strftime format
+   * @param fmt Format string for @c strftime
+   * @return Date in target format
+   */
+  std::string format(const char *fmt) const;
+
   /** @brief Convert to day number
    * @return Day number
    */
@@ -126,22 +149,6 @@ public:
    */
   static int monthLength(int y, int m);
 
-private:
-  /** @brief Test for a leap year
-   * @return @c true iff @ref y is a leap year
-   */
-  inline bool isLeapYear() const {
-    return isLeapYear(y);
-  }
-
-  /** @brief Test for a leap year
-   * @param y Year
-   * @return @c true iff @ref y is a leap year
-   */
-  static inline bool isLeapYear(int y) {
-    return y % 4 == 0 && (y % 100 != 0 || y % 400 == 0);
-  }
-
   /** @brief Year
    *
    * This is the Gregorian year; unlike the C library it is not offset by
@@ -161,6 +168,22 @@ private:
    */
   int d;
 
+private:
+  /** @brief Test for a leap year
+   * @return @c true iff @ref y is a leap year
+   */
+  inline bool isLeapYear() const {
+    return isLeapYear(y);
+  }
+
+  /** @brief Test for a leap year
+   * @param y Year
+   * @return @c true iff @ref y is a leap year
+   */
+  static inline bool isLeapYear(int y) {
+    return y % 4 == 0 && (y % 100 != 0 || y % 400 == 0);
+  }
+
   /** @brief Day number at start of month
    *
    * The first of January is day 0.
