@@ -16,14 +16,21 @@
 #include "rsbackup.h"
 #include "Document.h"
 #include "Conf.h"
+#include "Device.h"
+#include "Backup.h"
+#include "Volume.h"
+#include "Host.h"
 #include "Command.h"
 #include "IO.h"
 #include "Database.h"
 #include "Report.h"
 #include "Utils.h"
+#include "Subprocess.h"
+#include "Errors.h"
 #include <cmath>
 #include <cstdlib>
 #include <stdexcept>
+#include <sstream>
 #include <boost/range/adaptor/reversed.hpp>
 
 // Split up a color into RGB components
@@ -53,25 +60,66 @@ unsigned Report::pickColor(unsigned zero, unsigned one, double param) {
   return packColor(resultRgb);
 }
 
+void Report::compute() {
+  backups_missing = 0;
+  backups_partial = 0;
+  backups_out_of_date = 0;
+  backups_failed = 0;
+  devices_unknown = config.unknownDevices.size();
+  hosts_unknown = config.unknownHosts.size();
+  volumes_unknown = 0;
+  for(auto &h: config.hosts) {
+    const Host *host = h.second;
+    volumes_unknown += host->unknownVolumes.size();
+    for(auto &v: host->volumes) {
+      const Volume *volume = v.second;
+      bool out_of_date = true;
+      size_t devices_used = 0;
+      for(auto &d: config.devices) {
+        const Device *device = d.second;
+        auto perDevice = volume->findDevice(device->name);
+        if(perDevice && perDevice->count) {
+          // At least one successful backup exists...
+          int newestAge = Date::today() - perDevice->newest;
+          if(newestAge <= volume->maxAge)
+            out_of_date = false;        // ...and it's recent enough
+          ++devices_used;
+        }
+        // Look for the most recent attempt at this device
+        const Backup *most_recent_backup = nullptr;
+        for(const Backup *b: boost::adaptors::reverse(volume->backups)) {
+          if(b->getStatus() == COMPLETE && b->deviceName == device->name) {
+            most_recent_backup = b;
+            break;
+          }
+        }
+        if(most_recent_backup && most_recent_backup->rc != 0)
+          ++backups_failed;             // most recent backup failed
+      }
+      if(devices_used < config.devices.size()) { // some device lacks a backup
+        if(devices_used == 0)
+          ++backups_missing;            // no device has a backup
+        else
+          ++backups_partial;            // only some devices have a backup
+      } else if(out_of_date)
+        ++backups_out_of_date;          // no device has a recent enough backup
+    }
+  }
+}
+
 // Generate the list of warnings
-void Report::reportWarnings() {
+void Report::warnings() {
   char buffer[1024];
   Document::List *l = new Document::List();
-  for(auto &d: config.unknownDevices) {
+  for(auto &d: config.unknownDevices)
     l->entry("Unknown device " + d);
-    ++devices_unknown;
-  }
-  for(auto &h: config.unknownHosts) {
+  for(auto &h: config.unknownHosts)
     l->entry("Unknown host " + h);
-    ++hosts_unknown;
-  }
   for(auto &h: config.hosts) {
     const std::string &hostName = h.first;
     Host *host = h.second;
-    for(auto &v: host->unknownVolumes) {
+    for(auto &v: host->unknownVolumes)
       l->entry("Unknown volume " + v + " on host " + hostName);
-      ++volumes_unknown;
-    }
   }
   if(backups_missing) {
     snprintf(buffer, sizeof buffer, "WARNING: %d volumes have no backups.",
@@ -93,15 +141,22 @@ void Report::reportWarnings() {
              backups_failed);
     l->entry(buffer);
   }
-  if(l->nodes.size() > 0) {
-    d.heading("Warnings", 2);
-    d.append(l);
-  } else
-    delete l;
+  d.append(l);
+}
+
+int Report::warningCount() const {
+  int warnings = config.unknownDevices.size() + config.unknownHosts.size();
+  for(auto &h: config.hosts)
+    warnings += h.second->unknownVolumes.size();
+  if(backups_missing) ++warnings;
+  if(backups_partial) ++warnings;
+  if(backups_out_of_date) ++warnings;
+  if(backups_failed) ++warnings;
+  return warnings;
 }
 
 // Generate the summary table
-Document::Table *Report::reportSummary() {
+void Report::summary() {
   Document::Table *t = new Document::Table();
 
   t->addHeadingCell(new Document::Cell("Host", 1, 3));
@@ -122,15 +177,15 @@ Document::Table *Report::reportSummary() {
   t->newRow();
 
   for(auto &h: config.hosts) {
-    Host *host = h.second;
+    const Host *host = h.second;
     t->addCell(new Document::Cell(host->name, 1, host->volumes.size()))
       ->style = "host";
     for(auto &v: host->volumes) {
-      Volume *volume = v.second;
+      const Volume *volume = v.second;
       // See if every device has a backup
       bool missingDevice = false;
-      for(auto &d: config.devices) {
-        Device *device = d.second;
+      for(const auto &d: config.devices) {
+        const Device *device = d.second;
         if(!contains(volume->perDevice, device->name))
           missingDevice = true;
       }
@@ -141,54 +196,34 @@ Document::Table *Report::reportSummary() {
                                     : "none"));
       t->addCell(new Document::Cell(new Document::String(volume->completed)))
         ->style = missingDevice ? "bad" : "good";
-      bool out_of_date = true;
-      size_t devices_used = 0;
-      for(auto &d: config.devices) {
+      for(const auto &d: config.devices) {
         const Device *device = d.second;
-        Volume::PerDevice &perDevice = volume->perDevice[device->name];
-        if(perDevice.count) {
+        auto perDevice = volume->findDevice(device->name);
+        int perDeviceCount = perDevice ? perDevice->count : 0;
+        if(perDeviceCount) {
           // At least one successful backups
           Document::Cell *c
-            = t->addCell(new Document::Cell(perDevice.newest.toString()));
-          int newestAge = Date::today() - perDevice.newest;
+            = t->addCell(new Document::Cell(perDevice->newest.toString()));
+          int newestAge = Date::today() - perDevice->newest;
           if(newestAge <= volume->maxAge) {
             double param = (pow(2, (double)newestAge / volume->maxAge) - 1) / 2.0;
             c->bgcolor = pickColor(config.colorGood, config.colorBad, param);
-            out_of_date = false;
           } else {
             c->style = "bad";
           }
-          ++devices_used;
         } else {
           // No succesful backups!
           t->addCell(new Document::Cell("none"))
             ->style = "bad";
         }
-        t->addCell(new Document::Cell(new Document::String(perDevice.count)))
-          ->style = perDevice.count ? "good" : "bad";
-        // Look for the most recent attempt at this device
-        const Backup *most_recent_backup = nullptr;
-        for(const Backup *b: boost::adaptors::reverse(volume->backups)) {
-          if(b->getStatus() == COMPLETE && b->deviceName == device->name) {
-            most_recent_backup = b;
-            break;
-          }
-        }
-        if(most_recent_backup && most_recent_backup->rc != 0)
-          ++backups_failed;
+        t->addCell(new Document::Cell(new Document::String(perDeviceCount)))
+          ->style = perDeviceCount ? "good" : "bad";
       }
-      if(devices_used < config.devices.size()) {
-        if(devices_used == 0)
-          ++backups_missing;
-        else
-          ++backups_partial;
-      } else if(out_of_date)
-        ++backups_out_of_date;
       t->newRow();
     }
   }
 
-  return t;
+  d.append(t);
 }
 
 // Return true if this is a suitable log for the report
@@ -219,9 +254,9 @@ bool Report::suitableLog(const Volume *volume, const Backup *backup) {
 }
 
 // Generate the report of backup logfiles for a volume
-void Report::reportLogs(const Volume *volume) {
+void Report::logs(const Volume *volume) {
   Document::LinearContainer *lc = nullptr;
-  Host *host = volume->parent;
+  const Host *host = volume->parent;
   // Backups for a volume are ordered primarily by date and secondarily by
   // device.  The most recent backups are the most interesting so they are
   // displayed in reverse.
@@ -257,19 +292,24 @@ void Report::reportLogs(const Volume *volume) {
 }
 
 // Generate the report of backup logfiles for everything
-void Report::reportLogs() {
+void Report::logs() {
   // Sort by host/volume first, then date, device *last*
   for(auto &h: config.hosts) {
-    Host *host = h.second;
+    const Host *host = h.second;
     for(auto &v: host->volumes) {
-      Volume *volume = v.second;
-      reportLogs(volume);
+      const Volume *volume = v.second;
+      logs(volume);
     }
   }
 }
 
 // Generate the report of pruning logfiles
-Document::Node *Report::reportPruneLogs() {
+void Report::pruneLogs(const std::string &days) {
+  int ndays = DEFAULT_PRUNE_REPORT_AGE;
+  if(days.size())
+    ndays = parseInteger(days, 0, INT_MAX);
+  if(config.reportPruneLogs)
+    ndays = config.reportPruneLogs;
   Document::Table *t = new Document::Table();
 
   t->addHeadingCell(new Document::Cell("Created", 1, 1));
@@ -280,7 +320,7 @@ Document::Node *Report::reportPruneLogs() {
   t->addHeadingCell(new Document::Cell("Reason", 1, 1));
   t->newRow();
 
-  const int64_t cutoff = Date::now() - 86400 * config.reportPruneLogs;
+  const int64_t cutoff = Date::now() - 86400 * ndays;
   Database::Statement stmt(config.getdb(),
                            "SELECT host,volume,device,time,pruned,log"
                            " FROM backup"
@@ -312,48 +352,82 @@ Document::Node *Report::reportPruneLogs() {
     t->newRow();
   }
 
-  return t;
+  d.append(t);
+}
+
+void Report::historyGraph() {
+  std::string history_png;
+  std::string rg = Subprocess::pathSearch("rsbackup-graph");
+  if(rg.size() == 0)
+    return;
+  std::vector<std::string> cmd = {
+    "rsbackup-graph",
+    "-c", configPath,
+    "-D", database,
+    "-o-",
+  };
+  if(debug)
+    cmd.push_back("-d");
+  Subprocess sp(cmd);
+  sp.capture(1, &history_png);
+  sp.runAndWait();
+  if(history_png.size()) {
+    std::stringstream ss;
+    ss << "data:image/png;base64,";
+    write_base64(ss, history_png);
+    Document::Image *image = new Document::Image(ss.str());
+    image->style = "history";
+    d.append(image);
+  }
+}
+
+void Report::section(const std::string &n) {
+  std::string name = n, value, condition;
+  size_t colon = name.find("?");
+  if(colon != std::string::npos) {
+    condition.assign(name, colon + 1, std::string::npos);
+    name.erase(colon);
+    if(condition == "warnings") {
+      if(!warningCount())
+        return;
+    } else
+      throw SyntaxError("unrecognized report condition '" + condition + "'");
+    // Update Conf.cc and rsbackup.5 if any new conditionss
+  }
+  colon = name.find(":");
+  if(colon != std::string::npos) {
+    value = substitute(name, colon + 1, std::string::npos);
+    name.erase(colon);
+  }
+  if(name == "warnings") warnings();
+  else if(name == "summary") summary();
+  else if(name == "logs") logs();
+  else if(name == "prune-logs") pruneLogs(value);
+  else if(name == "history-graph") historyGraph();
+  else if(name == "h1") d.heading(value, 1);
+  else if(name == "h2") d.heading(value, 2);
+  else if(name == "h3") d.heading(value, 3);
+  else if(name == "p") d.para(value);
+  else if(name == "title") d.title = value;
+  else throw SyntaxError("unrecognized report name '" + name + "'");
+  // Update Conf.cc and rsbackup.5 if any new names added
 }
 
 // Generate the full report
 void Report::generate() {
-  backups_missing = 0;
-  backups_partial = 0;
-  backups_out_of_date = 0;
-  backups_failed = 0;
-  devices_unknown = 0;
-  hosts_unknown = 0;
-  volumes_unknown = 0;
-
-  d.title = "Backup report (" + Date::today().toString() + ")";
-  d.heading(d.title);
-
-  /* Generating the summary has a side effect of filling in the summary
-   * counters */
-  Document::Table *report = reportSummary();
-
-  // Unknown objects ----------------------------------------------------------
-  reportWarnings();
-
-  // Summary table ------------------------------------------------------------
-  d.heading("Summary", 2);
-  d.append(report);
-
-  // Logfiles -----------------------------------------------------------------
-  d.heading("Logfiles", 2);
-  reportLogs();
-
-  // Prune logs ---------------------------------------------------------------
-  d.heading("Pruning logs", 3);         // TODO anchor
-  d.append(reportPruneLogs());
-
-  // Generation time ----------------------------------------------------------
-  Document::Paragraph *p = d.para("Generated ");
-  if(getenv("RSBACKUP_TODAY"))
-    p->append(new Document::String("<timestamp>"));
-  else {
+  if(::setenv("RSBACKUP_DATE",
+              Date::today().toString().c_str(), 1/*overwrite*/))
+    throw SystemError("setenv", errno);
+  if(getenv("RSBACKUP_TODAY")) {
+    if(::setenv("RSBACKUP_CTIME", "<timestamp>", 1/*overwrite*/))
+      throw SystemError("setenv", errno);
+  } else {
     time_t now;
     time(&now);
-    p->append(new Document::String(ctime(&now)));
+    if(::setenv("RSBACKUP_CTIME", ctime(&now), 1/*overwrite*/))
+      throw SystemError("setenv", errno);
   }
+  compute();
+  for(const auto &s: config.report)
+    section(s);
 }
